<?php

namespace App\Adapter;

use FilesystemIterator;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
use SplFileInfo;
use wapmorgan\Mp3Info\Mp3Info;
use function Lambdish\Phunctional\filter;
use function Lambdish\Phunctional\map;
use function Lambdish\Phunctional\pipe;

class SongScanner
{

    private string $rootDir;

    public function __construct(string $rootDir) {
        $this->rootDir = $rootDir;
    }

    /**
     * @return ScannedSong[]
     */
    public function scan(): array {
        $it = new RecursiveDirectoryIterator(
            $this->rootDir,
            FilesystemIterator::FOLLOW_SYMLINKS | FilesystemIterator::SKIP_DOTS
        );

        $pipe = pipe([$this, 'isMp3'], [$this, 'toScannedFile']);
        return $pipe(new RecursiveIteratorIterator($it));
    }

    public static function isMp3(iterable $files): iterable {
        return filter(function (SplFileInfo $file, string $_): bool {
            return $file->isFile() && strtolower($file->getExtension()) === 'mp3';
        }, $files);
    }

    public static function toScannedFile(iterable $files): iterable {
        return map(
            function (SplFileInfo $file, string $path): ScannedSong {
                $song = new ScannedSong();
                $song->file = $file;
                $song->path = $path;

                $id3 = new Mp3Info($path, true);
                $song->artist = $id3->tags['artist'] ?: '';
                $song->album = $id3->tags['album'] ?: '';
                $song->title = $id3->tags['song'] ?: '';
                $song->duration = $id3->duration ?: 0;

                return $song;
            }, $files);
    }

}
